@@ -1,6 +1,6 @@
 
 /* Copyright (c) 2004 - 2006 Derek Foreman, Ben Jansens
-   Copyright (c) 2006 - 2010 Thomas Schmitt <scdbackup@gmx.net>
+   Copyright (c) 2006 - 2011 Thomas Schmitt <scdbackup@gmx.net>
    Provided under GPL version 2 or later.
 */
 
@@ -170,6 +170,8 @@ struct burn_track *burn_track_create(void)
 	/* ts A61031 */
 	t->open_ended = 0;
 	t->track_data_done = 0;
+	/* ts B10103 */
+	t->end_on_premature_eoi = 0;
 
 	t->postgap = 0;
 	t->pregap1 = 0;
