@@ -2,7 +2,7 @@
 /*
   Class struct of libisoburn.
 
-  Copyright 2007 - 2010 Vreixo Formoso Lopes <metalpain2002@yahoo.es>
+  Copyright 2007 - 2011 Vreixo Formoso Lopes <metalpain2002@yahoo.es>
                     and Thomas Schmitt <scdbackup@gmx.net>
   Provided under GPL version 2 or later.
 */
@@ -12,7 +12,13 @@
 
 
 /* for uint8_t */
+#ifdef HAVE_STDINT_H
 #include <stdint.h>
+#else
+#ifdef HAVE_INTTYPES_H
+#include <inttypes.h>
+#endif
+#endif
 
 /* For emulated TOC of overwriteable media.
    Provides minimal info for faking a struct burn_toc_entry.
@@ -370,7 +376,7 @@ struct isoburn_imgen_opts {
     unsigned int session_md5 :1;
 
     /* Produce and write MD5 checksums for each single IsoFile.
-       See parameter files of iso_write_opts_set_record_md5().
+       See parameter "files" of iso_write_opts_set_record_md5().
      */
     unsigned int file_md5 :2;
 
@@ -379,6 +385,13 @@ struct isoburn_imgen_opts {
      */
     unsigned int no_emul_toc :1;
 
+    /* For empty files, symbolic links, and devices use the old ECMA-119 block
+       addresses in the range [0,31] rather than the address of the dedicated
+       empty block.
+    */
+    unsigned int old_empty :1;
+
+
     /* relaxed constraints */
 
     /*
