@@ -224,7 +224,7 @@ void isoburn_version(int *major, int *minor, int *micro);
 */
 #define isoburn_libisofs_req_major  1
 #define isoburn_libisofs_req_minor  0
-#define isoburn_libisofs_req_micro  0 
+#define isoburn_libisofs_req_micro  2 
 
 /** The minimum version of libburn to be used with this version of libisoburn
     at compile time.
@@ -232,7 +232,7 @@ void isoburn_version(int *major, int *minor, int *micro);
 */
 #define isoburn_libburn_req_major  1
 #define isoburn_libburn_req_minor  0
-#define isoburn_libburn_req_micro  0
+#define isoburn_libburn_req_micro  2
 
 /** The minimum compile time requirements of libisoburn towards libjte are
     the same as of a suitable libisofs towards libjte.
@@ -287,7 +287,7 @@ int isoburn_libburn_req(int *major, int *minor, int *micro);
 */
 #define isoburn_header_version_major  1
 #define isoburn_header_version_minor  0
-#define isoburn_header_version_micro  0
+#define isoburn_header_version_micro  2
 /** Note:
     Above version numbers are also recorded in configure.ac because libtool
     wants them as parameters at build time.
@@ -1006,6 +1006,15 @@ int isoburn_igopt_get_level(struct isoburn_imgen_opts *o, int *level);
                  and that the write thread will be cancelled by
                  isoburn_cancel_prepared_write() before actual image writing
                  occurs. Without this, cancellation can cause a MISHAP event.
+          bit11= old_empty
+                 @since 1.0.2
+                 Let symbolic links and device files point to block 0, and let
+                 empty data files point to the address of the Volume Descriptor
+                 Set Terminator. This was done by libisofs in the past.
+                 By default there is now a single dedicated block of zero bytes
+                 after the end of the directory trees, of which the address
+                 is used for all files without own content.
+
     @return 1 success, <=0 failure
 */
 #define isoburn_igopt_rockridge         1
@@ -1018,6 +1027,7 @@ int isoburn_igopt_get_level(struct isoburn_imgen_opts *o, int *level);
 #define isoburn_igopt_file_stability  256
 #define isoburn_igopt_no_emul_toc     512
 #define isoburn_igopt_will_cancel    1024
+#define isoburn_igopt_old_empty      2048
 int isoburn_igopt_set_extensions(struct isoburn_imgen_opts *o, int ext);
 int isoburn_igopt_get_extensions(struct isoburn_imgen_opts *o, int *ext);
 
