@@ -1,7 +1,7 @@
 /*
  * Copyright (c) 2007 Vreixo Formoso
  * Copyright (c) 2007 Mario Danic
- * Copyright (c) 2009 - 2010 Thomas Schmitt
+ * Copyright (c) 2009 - 2011 Thomas Schmitt
  *
  * This file is part of the libisofs project; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License version 2 
@@ -141,7 +141,7 @@ static int show_chunk_to_jte(Ecma119Image *target, char *buf, int count)
 static
 int need_version_number(Ecma119Image *t, Ecma119Node *n)
 {
-    if (t->omit_version_numbers & 1) {
+    if ((t->omit_version_numbers & 1) || t->untranslated_name_len > 0) {
         return 0;
     }
     if (n->type == ECMA119_DIR || n->type == ECMA119_PLACEHOLDER) {
@@ -188,9 +188,9 @@ size_t calc_dir_size(Ecma119Image *t, Ecma119Node *dir, size_t *ce)
     /* size of "." and ".." entries */
     len = 34 + 34;
     if (t->rockridge) {
-        len += rrip_calc_len(t, dir, 1, 255 - 34, &ce_len);
+        len += rrip_calc_len(t, dir, 1, 34, &ce_len);
         *ce += ce_len;
-        len += rrip_calc_len(t, dir, 2, 255 - 34, &ce_len);
+        len += rrip_calc_len(t, dir, 2, 34, &ce_len);
         *ce += ce_len;
     }
 
@@ -203,7 +203,7 @@ size_t calc_dir_size(Ecma119Image *t, Ecma119Node *dir, size_t *ce)
         for (section = 0; section < nsections; ++section) {
             size_t dirent_len = calc_dirent_len(t, child);
             if (t->rockridge) {
-                dirent_len += rrip_calc_len(t, child, 0, 255 - dirent_len, &ce_len);
+                dirent_len += rrip_calc_len(t, child, 0, dirent_len, &ce_len);
                 *ce += ce_len;
             }
             remaining = BLOCK_SIZE - (len % BLOCK_SIZE);
@@ -329,6 +329,9 @@ int ecma119_writer_compute_data_blocks(IsoImageWriter *writer)
                              image start and not for the partition */;
 
     }
+
+    target->tree_end_block = target->curblock;
+
     return ISO_SUCCESS;
 }
 
@@ -380,11 +383,14 @@ void write_one_dir_record(Ecma119Image *t, Ecma119Node *node, int file_id,
         multi_extend = (node->info.file->nsections - 1 == extent) ? 0 : 1;
     } else {
         /*
-         * for nodes other than files and dirs, we set both
-         * len and block to 0
+         * for nodes other than files and dirs, we set len to 0, and
+         * the content block address to a dummy value.
          */
         len = 0;
-        block = 0;
+        if (! t->old_empty)
+            block = t->empty_file_block;
+        else
+            block = 0;
     }
 
     /*
@@ -594,7 +600,7 @@ int write_one_dir(Ecma119Image *t, Ecma119Node *dir, Ecma119Node *parent)
 
     /* write the "." and ".." entries first */
     if (t->rockridge) {
-        ret = rrip_get_susp_fields(t, dir, 1, 255 - 32, &info);
+        ret = rrip_get_susp_fields(t, dir, 1, 34, &info);
         if (ret < 0) {
             return ret;
         }
@@ -604,7 +610,7 @@ int write_one_dir(Ecma119Image *t, Ecma119Node *dir, Ecma119Node *parent)
     buf += len;
 
     if (t->rockridge) {
-        ret = rrip_get_susp_fields(t, dir, 2, 255 - 32, &info);
+        ret = rrip_get_susp_fields(t, dir, 2, 34, &info);
         if (ret < 0) {
             return ret;
         }
@@ -630,7 +636,7 @@ int write_one_dir(Ecma119Image *t, Ecma119Node *dir, Ecma119Node *parent)
 
             /* get the SUSP fields if rockridge is enabled */
             if (t->rockridge) {
-                ret = rrip_get_susp_fields(t, child, 0, 255 - len, &info);
+                ret = rrip_get_susp_fields(t, child, 0, len, &info);
                 if (ret < 0) {
                     return ret;
                 }
@@ -871,6 +877,7 @@ int ecma119_writer_write_data(IsoImageWriter *writer)
 {
     int ret;
     Ecma119Image *t;
+    uint32_t curblock;
 
     if (writer == NULL) {
         return ISO_ASSERT_FAILURE;
@@ -888,6 +895,20 @@ int ecma119_writer_write_data(IsoImageWriter *writer)
         if (ret < 0) 
             return ret;
     }
+
+    curblock = (t->bytes_written / 2048) + t->ms_block;
+    if (curblock != t->tree_end_block) {
+        char msg[100];
+
+        sprintf(msg,
+                "Calculated and written ECMA-119 tree end differ: %lu <> %lu",
+                (unsigned long) t->tree_end_block,
+                (unsigned long) curblock);
+        iso_msgs_submit(0, msg, 0, "WARNING", 0);
+
+        t->tree_end_block = 1;/* Mark for harsher reaction at end of writing */
+    }
+
     return ISO_SUCCESS;
 }
 
@@ -1135,6 +1156,8 @@ int write_vol_desc_terminator(Ecma119Image *target)
     uint8_t buf[BLOCK_SIZE];
     struct ecma119_vol_desc_terminator *vol;
 
+    memset(buf, 0, BLOCK_SIZE);
+
     vol = (struct ecma119_vol_desc_terminator *) buf;
 
     vol->vol_desc_type[0] = 255;
@@ -1209,7 +1232,7 @@ int write_head_part2(Ecma119Image *target, int *write_count, int flag)
         return ISO_SUCCESS;
 
     /* Write multi-session padding up to target->partition_offset + 16 */
-    memset(buf, 0, 2048);
+    memset(buf, 0, BLOCK_SIZE);
     for(; *write_count < target->partition_offset + 16; (*write_count)++) {
         res = iso_write(target, buf, BLOCK_SIZE);
         if (res < 0)
@@ -1400,6 +1423,12 @@ void *write_function(void *arg)
        Eventually free target */
     ecma119_image_free(target);
 
+    if (target->tree_end_block == 1) {
+        iso_msgs_submit(0,
+    "Image is most likely damaged. Calculated/written block address mismatch.",
+                        0, "WARNING", 0);
+    }
+
 #ifdef Libisofs_with_pthread_exiT
     pthread_exit(NULL);
 #else
@@ -1575,6 +1604,9 @@ int ecma119_image_new(IsoImage *src, IsoWriteOpts *opts, Ecma119Image **img)
     target->hardlinks = opts->hardlinks;
     target->aaip = opts->aaip;
     target->always_gmt = opts->always_gmt;
+    target->old_empty = opts->old_empty;
+    target->untranslated_name_len = opts->untranslated_name_len;
+    target->allow_dir_id_ext = opts->allow_dir_id_ext;
     target->omit_version_numbers = opts->omit_version_numbers
                                  | opts->max_37_char_filenames;
     target->allow_deep_paths = opts->allow_deep_paths;
@@ -1731,6 +1763,7 @@ int ecma119_image_new(IsoImage *src, IsoWriteOpts *opts, Ecma119Image **img)
     target->mipsel_p_filesz = 0;
 
     target->empty_file_block = 0;
+    target->tree_end_block = 0;
 
     target->wthread_is_running = 0;
 
@@ -1918,8 +1951,15 @@ int ecma119_image_new(IsoImage *src, IsoWriteOpts *opts, Ecma119Image **img)
         if (i == el_torito_writer_index)
     continue;
 
-        /* Exposing address of data start to IsoWriteOpts */
+        /* Exposing address of data start to IsoWriteOpts iand memorizing
+           this address for for all files which have no block address: 
+           symbolic links, device files, empty data files.
+           filesrc_writer_compute_data_blocks() and filesrc_writer_write_data()
+           will account resp. write this single block. 
+        */
         if (i == file_src_writer_index) {
+            if (! target->old_empty)
+                target->empty_file_block = target->curblock;
             opts->data_start_lba = target->curblock;
         }
 
@@ -2266,6 +2306,8 @@ int iso_write(Ecma119Image *target, void *buf, size_t count)
         /* reader cancelled */
         return ISO_CANCELED;
     }
+    if (ret < 0)
+        return ret;
     if (target->checksum_ctx != NULL) {
         /* Add to image checksum */
         target->checksum_counter += count;
@@ -2277,7 +2319,7 @@ int iso_write(Ecma119Image *target, void *buf, size_t count)
         return ret;
 
     /* total size is 0 when writing the overwrite buffer */
-    if (ret > 0 && (target->total_size != (off_t) 0)){
+    if (target->total_size != (off_t) 0){
         unsigned int kbw, kbt;
         int percent;
 
@@ -2294,7 +2336,7 @@ int iso_write(Ecma119Image *target, void *buf, size_t count)
         }
     }
 
-    return ret;
+    return ISO_SUCCESS;
 }
 
 int iso_write_opts_new(IsoWriteOpts **opts, int profile)
@@ -2363,6 +2405,9 @@ int iso_write_opts_new(IsoWriteOpts **opts, int profile)
         wopts->appended_partitions[i] = NULL;
     wopts->ascii_disc_label[0] = 0;
     wopts->will_cancel = 0;
+    wopts->allow_dir_id_ext = 0;
+    wopts->old_empty = 0;
+    wopts->untranslated_name_len = 0;
 
     *opts = wopts;
     return ISO_SUCCESS;
@@ -2451,6 +2496,40 @@ int iso_write_opts_set_aaip(IsoWriteOpts *opts, int enable)
     return ISO_SUCCESS;
 }
 
+int iso_write_opts_set_old_empty(IsoWriteOpts *opts, int enable)
+{
+    if (opts == NULL) {
+        return ISO_NULL_POINTER;
+    }
+    opts->old_empty = enable ? 1 : 0;
+    return ISO_SUCCESS;
+}
+
+int iso_write_opts_set_untranslated_name_len(IsoWriteOpts *opts, int len)
+{
+    if (opts == NULL) {
+        return ISO_NULL_POINTER;
+    }
+    if (len == -1)
+        opts->untranslated_name_len = ISO_UNTRANSLATED_NAMES_MAX;
+    else if(len == 0)
+        opts->untranslated_name_len = 0;
+    else if(len > ISO_UNTRANSLATED_NAMES_MAX || len < 0)
+        return ISO_WRONG_ARG_VALUE;
+    else
+        opts->untranslated_name_len = len;
+    return opts->untranslated_name_len;
+}
+
+int iso_write_opts_set_allow_dir_id_ext(IsoWriteOpts *opts, int allow)
+{
+    if (opts == NULL) {
+        return ISO_NULL_POINTER;
+    }
+    opts->allow_dir_id_ext = allow ? 1 : 0;
+    return ISO_SUCCESS;
+}
+
 int iso_write_opts_set_omit_version_numbers(IsoWriteOpts *opts, int omit)
 {
     if (opts == NULL) {
