@@ -25,7 +25,14 @@
 #include <sys/time.h>
 #include <time.h>
 #include <errno.h>
+
+#ifdef HAVE_STDINT_H
 #include <stdint.h>
+#else
+#ifdef HAVE_INTTYPES_H
+#include <inttypes.h>
+#endif
+#endif
 
 #ifdef Xorriso_standalonE
 
@@ -125,6 +132,7 @@ int Xorriso_new(struct XorrisO ** xorriso,char *progname, int flag)
  m->do_aaip= 0;
  m->do_md5= 0;
  m->no_emul_toc= 0;
+ m->do_old_empty= 0;
  m->scdbackup_tag_name[0]= 0;
  m->scdbackup_tag_time[0]= 0;
  m->scdbackup_tag_written[0]= 0;
@@ -355,6 +363,8 @@ int Xorriso_new(struct XorrisO ** xorriso,char *progname, int flag)
 
  m->perm_stack= NULL;
 
+ m->update_flags= 0;
+
  m->result_line[0]= 0;
  m->result_line_counter= 0;
  m->result_page_counter= 0;
@@ -437,6 +447,10 @@ int Xorriso_destroy(struct XorrisO **xorriso, int flag)
  m= *xorriso;
  if(m==NULL)
    return(0);
+
+ /* Give up drives and image to unref all connected xorriso objects */
+ Xorriso_give_up_drive(m, 3);
+
  if(m->in_charset!=NULL)
    free(m->in_charset);
  if(m->out_charset!=NULL)
