@@ -1132,8 +1132,8 @@ int Xorriso_findx_action(struct XorrisO *xorriso, struct FindjoB *job,
    if(ret<xorriso->find_compare_result)
      xorriso->find_compare_result= ret;
    if(ret==0) {
-     uret= Xorriso_update_interpreter(xorriso, NULL, compare_result,
-                                      abs_path, iso_path, (flag&1)<<2);
+     uret= Xorriso_update_interpreter(xorriso, NULL, NULL, compare_result,
+                                      abs_path, iso_path, ((flag&1)<<2) | 2);
      if(uret<=0)
        ret= 0;
    }
