@@ -533,7 +533,8 @@ int Xorriso_genisofs_count_args(struct XorrisO *xorriso, int argc, char **argv,
     "-no-mac-files", "-chrp-boot",
     "--hardlinks", "--acl", "--xattr", "--md5", "--for_backup",
     "--protective-msdos-label", "--boot-catalog-hide", "--no-emul-toc",
-    "--emul-toc", "-disallow_dir_id_ext",
+    "--emul-toc", "-disallow_dir_id_ext", "--old-empty",
+    "--old-root-no-md5", "--old-root-devno", "--old-root-no-ino",
     ""
  };
  static char arg1_options[][41]= {
@@ -618,16 +619,17 @@ int Xorriso_genisofs_ignore(struct XorrisO *xorriso, char *whom,
  static char ignored_arg0_options[][41]= {
    "-allow-leading-dots", "-ldots", "-allow-multidot",
    "-cache-inodes", "-no-cache-inodes", "-check-oldnames", "-D",
-   "-L", "-max-iso9660-filenames", "-nobak",
+   "-disable-deep-relocation",
+   "-L", "-nobak",
    "-no-bak", "-force-rr", "-relaxed-filenames", "-T",
    "-no-iso-translate",
    ""
  };
  static char ignored_arg1_options[][41]= {
-   "-check-session", "-hide-hfs", "-hide-hfs-list", "-root",
-   "-old-root", "-table-name", "-volset-seqno", "-volset-size",
+   "-check-session", "-hide-hfs", "-hide-hfs-list",
+   "-table-name", "-volset-seqno", "-volset-size",
    ""
- };
+ };                                               /* "-root", "-old-root", */
  int k, idx_offset= 0;
  char sfe[5*SfileadrL];
 
@@ -692,6 +694,7 @@ int Xorriso_genisofs_help(struct XorrisO *xorriso, int flag)
 "  -allow-lowercase            Allow lower case characters in addition to the current character set (violates ISO9660)",
 "  -d, -omit-period            Omit trailing periods from filenames (violates ISO9660)",
 "  -l, -full-iso9660-filenames Allow full 31 character filenames for ISO9660 names",
+"  -max-iso9660-filenames      Allow 37 character filenames for ISO9660 names (violates ISO9660)",
 "  -N, -omit-version-number    Omit version number from ISO9660 filename (violates ISO9660)",
 "  -o FILE, -output FILE       Set output file name",
 "  -m GLOBFILE, -exclude GLOBFILE",
@@ -722,10 +725,10 @@ int Xorriso_genisofs_help(struct XorrisO *xorriso, int flag)
 "  -publisher PUB              Set Volume publisher",
 "  -A ID, -appid ID            Set Application ID",
 "  -sysid ID                   Set System ID",
+"  -p PREP, -preparer PREP     Set Volume preparer",
 "  -abstract FILE              Set Abstract filename",
 "  -biblio FILE                Set Bibliographic filename",
 "  -copyright FILE             Set Copyright filename",
-"  -p PREP, -preparer PREP     Set Volume preparer",
 "  -jigdo-jigdo FILE           Produce a jigdo .jigdo file as well as the .iso",
 "  -jigdo-template FILE        Produce a jigdo .template file as well as the .iso",
 "  -jigdo-min-file-size SIZE   Minimum size for a file to be listed in the jigdo file", 
@@ -739,7 +742,7 @@ int Xorriso_genisofs_help(struct XorrisO *xorriso, int flag)
 "  -checksum_algorithm_iso alg1,alg2,...",
 "                              Specify the checksum types desired for the output image (in .jigdo)",
 "  -checksum_algorithm_template alg1,alg2,...",
-"                              Specify the checksum types desired for the output jigdo template"
+"                              Specify the checksum types desired for the output jigdo template",
 "  -b FILE, -eltorito-boot FILE",
 "                              Set El Torito boot image name",
 "  -eltorito-alt-boot          Start specifying alternative El Torito boot parameters",
@@ -753,6 +756,7 @@ int Xorriso_genisofs_help(struct XorrisO *xorriso, int flag)
 "  -no-emul-boot               Boot image is 'no emulation' image",
 "  -boot-info-table            Patch boot image with info table",
 "  -G FILE, -generic-boot FILE Set generic boot image name",
+"  --embedded-boot FILE        Alias of -G",
 "  --protective-msdos-label    Patch System Area by partition table",
 "  -partition_offset LBA       Make image mountable by first partition, too",
 "  -partition_sec_hd NUMBER    Define number of sectors per head",
@@ -773,12 +777,19 @@ int Xorriso_genisofs_help(struct XorrisO *xorriso, int flag)
 "  --stdio_sync on|off|number  Control forced output to disk files",
 "  --no-emul-toc               Save 64 kB size on random access output files",
 "  --emul-toc                  Multi-session history on such output files",
+"  --old-empty                 Use old style block addresses for empty files",
 "  -z, -transparent-compression",
 "                              Enable transparent compression of files",
+"  -root DIR                   Set root directory for all new files and directories",
+"  -old-root DIR               Set root directory in previous session that is searched for files",
+"  --old-root-no-md5           Do not record and use MD5 with -old-root",
+"  --old-root-no-ino           Do not use disk inode numbers with -old-root",
+"  --old-root-devno            Use disk device numbers with -old-root",
 "  -v, -verbose                Verbose",
 "  -version                    Print the current version",
 "@End_of_helptexT@"
 };
+
  char ra_text[80];
  int i;
 
@@ -882,15 +893,23 @@ int Xorriso_genisofs(struct XorrisO *xorriso, char *whom,
  int do_print_size= 0, fd, idx, iso_level= 1, emul_boot= 2;
  int option_b= 0, was_failure= 0, fret, lower_r= 0, zero= 0;
  int dir_mode= -1, file_mode= -1, count, partition_number, allow_dir_id_ext= 1;
+ int root_seen= 0;
  mode_t mode_and, mode_or;
  int with_boot_image= 0, with_cat_path= 0, with_emul_toc= 0;
+ int old_root_md5= 1, old_root_dev= 0, old_root_ino= 1;
  int *weight_list= NULL, weight_count= 0;
  int *delay_opt_list= NULL, delay_opt_count= 0;
  char sfe[5*SfileadrL], adr[SfileadrL+8], ra_text[80], pathspec[2*SfileadrL];
  char *ept, *add_pt, eff_path[SfileadrL], indev[SfileadrL+8], msc[80], *cpt;
- char *boot_path, partno_text[8];
+ char old_root[SfileadrL];
+ char *boot_path, partno_text[8], *iso_rr_pt, *disk_pt;
+ char *rm_merge_args[3];
+
+ /* >>> This should not be static. Why is it ? */
  static char *sort_weight_args[4]= {"", "-exec", "sort_weight", ""};
 
+ struct stat stbuf;
+
  strcpy(ra_text, xorriso->report_about_text);
 
  weight_list= TSOB_FELD(int, (argc / 3) + 1);
@@ -907,7 +926,7 @@ int Xorriso_genisofs(struct XorrisO *xorriso, char *whom,
 
  if(xorriso->boot_image_cat_path[0])
    with_cat_path= -1;
- adr[0]= indev[0]= msc[0]= 0;
+ adr[0]= indev[0]= msc[0]= old_root[0]= 0;
  for(i= 0; i<argc; i++) {
    ret= Xorriso_genisofs_ignore(xorriso, whom, argv, &i, 1);
    if(ret == 1)
@@ -1021,6 +1040,33 @@ int Xorriso_genisofs(struct XorrisO *xorriso, char *whom,
      if(ret <= 0)
        goto problem_handler_1;
 
+   } else if(strcmp(argv[i], "-root") == 0) {
+     if(i+1>=argc)
+       goto not_enough_args;
+     i++;
+     /* Always absolute */
+     strcpy(eff_path, "/");
+     if(Sfile_str(eff_path, argv[i], argv[i][0] != '/') <= 0)
+       {ret= -1; goto ex;}
+     strcpy(xorriso->wdi, eff_path);
+     root_seen= 1;
+
+   } else if(strcmp(argv[i], "-old-root") == 0) {
+     if(i+1>=argc)
+       goto not_enough_args;
+     i++;
+     /* Always absolute */
+     strcpy(old_root, "/");
+     if(Sfile_str(old_root, argv[i], argv[i][0] != '/') <= 0)
+       {ret= -1; goto ex;}
+
+   } else if(strcmp(argv[i], "--old-root-no-md5")==0) {
+     old_root_md5= 0;
+   } else if(strcmp(argv[i], "--old-root-devno")==0) {
+     old_root_dev= 1;
+   } else if(strcmp(argv[i], "--old-root-no-ino")==0) {
+     old_root_ino= 0;
+
    } else if(strcmp(argv[i], "--hardlinks")==0) {
      Xorriso_option_hardlinks(xorriso, "on", 0);
    } else if(strcmp(argv[i], "--acl")==0) {
@@ -1126,6 +1172,15 @@ illegal_c:;
    if(ret<=0)
      goto ex;
  }
+
+ if(old_root[0] || root_seen) {
+   Xorriso_option_md5(xorriso, old_root_md5 ? "on" : "off", 0);
+   Xorriso_option_disk_dev_ino(xorriso,
+                               old_root_dev && old_root_ino ? "on" :
+                               old_root_ino ? "ino_only" : "off", 0);
+   if(!old_root_ino)
+     Xorriso_option_hardlinks(xorriso, "without_update", 0);
+ }
  if(indev[0]) {
    ret= Xorriso_option_dev(xorriso, indev, 1);
    if(ret<=0)
@@ -1135,6 +1190,24 @@ illegal_c:;
  if(!was_other_option)
    {ret= 1; goto ex;}
 
+ if(old_root[0]) {
+   ret= Xorriso_iso_lstat(xorriso, old_root, &stbuf, 0);
+   if(ret >= 0) {
+     if(root_seen) {
+       ret= Xorriso_mkdir(xorriso, xorriso->wdi, 1 | 2);
+       if(ret < 0)
+         {ret= -(ret != -1); goto ex;}
+     } else {
+       strcpy(xorriso->wdi, "/");
+     }
+     if(strcmp(old_root, xorriso->wdi) != 0) {
+       ret= Xorriso_clone_under(xorriso, old_root, xorriso->wdi, 0);
+       if(ret <= 0)
+         goto ex;
+     }
+   }
+ }
+
  xorriso->padding= 300*1024;
 
  for(i= 0; i<argc; i++) {
@@ -1183,7 +1256,7 @@ not_enough_args:;
      xorriso->padding= 0;
    } else if(strcmp(argv[i], "-print-size")==0) {
      do_print_size= 1;
-   } else if(strcmp(argv[i], "-o")==0) {
+   } else if(strcmp(argv[i], "-o")==0 || strcmp(argv[i], "-output") == 0) {
      i++;
      /* was already handled in first argument scan */;
    } else if(strcmp(argv[i], "-M")==0  || strcmp(argv[i], "-dev")==0 ||
@@ -1323,7 +1396,7 @@ not_enough_args:;
    } else if(strcmp(argv[i], "-U") == 0 ||
              strcmp(argv[i], "-untranslated-filenames") == 0) {
      Xorriso_relax_compliance(xorriso,
-              "no_force_dots:long_paths:omit_version:full_ascii:lowercase", 0);
+       "no_force_dots:long_paths:long_names:omit_version:full_ascii:lowercase", 0);
    } else if(strcmp(argv[i], "-untranslated_name_len") == 0) {
      if(i+1>=argc)
        goto not_enough_args;
@@ -1336,8 +1409,9 @@ not_enough_args:;
              strcmp(argv[i], "-omit-version-number") == 0) {
      Xorriso_relax_compliance(xorriso, "omit_version", 0);
    } else if(strcmp(argv[i], "-l") == 0 ||
-             strcmp(argv[i], "-full-iso9660-filenames") == 0) {
-     Xorriso_relax_compliance(xorriso, "long_paths", 0);
+             strcmp(argv[i], "-full-iso9660-filenames") == 0 ||
+             strcmp(argv[i], "-max-iso9660-filenames") == 0) {
+     Xorriso_relax_compliance(xorriso, "long_names", 0);
    } else if(strcmp(argv[i], "-d") == 0 ||
              strcmp(argv[i], "-omit-period") == 0) {
      Xorriso_relax_compliance(xorriso, "no_force_dots", 0);
@@ -1352,6 +1426,16 @@ not_enough_args:;
        goto not_enough_args;
      i++;
      /* was already handled in first argument scan */;
+   } else if(strcmp(argv[i], "-root") == 0 ||
+             strcmp(argv[i], "-old-root") == 0) {
+     if(i+1>=argc)
+       goto not_enough_args;
+     i++;
+     /* was already handled in first argument scan */;
+   } else if(strcmp(argv[i], "--old-root-no-md5")==0 ||
+             strcmp(argv[i], "--old-root-devno")==0 ||
+             strcmp(argv[i], "--old-root-no-ino")==0) {
+     /* was already handled in first argument scan */;
    } else if(strcmp(argv[i], "-dir-mode") == 0) {
      if(i+1>=argc)
        goto not_enough_args;
@@ -1454,6 +1538,9 @@ not_enough_args:;
              strcmp(argv[i], "--no-emul-toc")==0) {
      /* was already handled in first argument scan */;
 
+   } else if(strcmp(argv[i], "--old-empty")==0) {
+     xorriso->do_old_empty= 1;
+
    } else if(strcmp(argv[i], "-disallow_dir_id_ext")==0) {
      /* was already handled in first argument scan */;
 
@@ -1493,13 +1580,41 @@ is_pathspec_2:;
        add_pt= pathspec;
      } else
        add_pt= argv[i];
-     mem_graft_points= xorriso->allow_graft_points;
-     xorriso->allow_graft_points= 1;
-     zero= 0;
-     ret= Xorriso_option_add(xorriso, 1, &add_pt, &zero, was_path<<1);
-     xorriso->allow_graft_points= mem_graft_points;
+
+     if(old_root[0]) {
+       /* Split pathspec */
+       ret= Fileliste__target_source_limit(pathspec, '=', &ept, 0);
+       if(ret > 0) {
+         *ept= 0;
+         iso_rr_pt= pathspec;
+         disk_pt= ept + 1;
+       } else {
+         iso_rr_pt= "/";
+         disk_pt= pathspec;
+       }
+       if(root_seen) {
+         strcpy(eff_path, iso_rr_pt);
+         iso_rr_pt= eff_path;
+         ret= Sfile_prepend_path(xorriso->wdi, iso_rr_pt, 0);
+         if(ret<=0) {
+           Xorriso_msgs_submit(xorriso, 0, "Effective path gets much too long",
+                               0, "FAILURE", 0);
+           goto problem_handler_2;
+         }
+       }
+       /* update_merge */
+       ret= Xorriso_option_update(xorriso, disk_pt, iso_rr_pt, 1 | 8 | 32);
+     } else {
+       mem_graft_points= xorriso->allow_graft_points;
+       xorriso->allow_graft_points= 1;
+       zero= 0;
+       ret= Xorriso_option_add(xorriso, 1, &add_pt, &zero,
+                               (was_path << 1) | (root_seen << 2));
+       xorriso->allow_graft_points= mem_graft_points;
+     }
      if(ret<=0)
        goto problem_handler_2;
+
      was_path= 1;
 
      /* Enforce odd mkisofs defaults on first pathspec */
@@ -1526,6 +1641,21 @@ problem_handler_2:;
  continue;
    goto ex;
  }
+
+ if(old_root[0]) {
+   /* Delete all visited nodes which were not found on disk */
+   if(root_seen)
+     rm_merge_args[0]= xorriso->wdi;
+   else
+     rm_merge_args[0]= "/";
+   rm_merge_args[1]= "-exec";
+   rm_merge_args[2]= "rm_merge";
+   zero= 0;
+   ret= Xorriso_option_find(xorriso, 3, rm_merge_args, &zero, 2);
+   if(ret<=0)
+     goto ex;
+ }
+
  if(lower_r) {
    static char *lower_r_args[3]= {"/", "-exec", "mkisofs_r"};
    zero= 0;
@@ -1727,7 +1857,7 @@ problem_handler_boot:;
 
  ret= !was_failure;
 ex:;
- if(was_path && !do_print_size)
+ if(was_path && (!do_print_size) && !old_root[0])
    Xorriso_pacifier_callback(xorriso, "files added", xorriso->pacifier_count,
                              xorriso->pacifier_total, "", 1);
  if(do_print_size && xorriso->volset_change_pending == 1)
