@@ -1,7 +1,7 @@
 
 /* xorriso - creates, loads, manipulates and burns ISO 9660 filesystem images.
 
-   Copyright 2007-2010 Thomas Schmitt, <scdbackup@gmx.net>
+   Copyright 2007-2011 Thomas Schmitt, <scdbackup@gmx.net>
 
    Provided under GPL version 2 or later.
 
@@ -118,6 +118,8 @@ int Xorriso_extf_destroy(struct XorrisO *xorriso, struct Xorriso_extF **filter,
  if(cmd != NULL) {
    if(cmd->refcount > 0)
      return(0);
+   if(cmd->path != NULL)
+     free(cmd->path);
    if(cmd->suffix != NULL)
      free(cmd->suffix);
    if(cmd->argv != NULL) {
@@ -128,6 +130,7 @@ int Xorriso_extf_destroy(struct XorrisO *xorriso, struct Xorriso_extF **filter,
    }
    if(cmd->name != NULL)
      free(cmd->name);
+   free((char *) cmd);
  }
  free((char *) *filter);
  *filter= NULL;
