@@ -1,7 +1,7 @@
 
 /* xorriso - creates, loads, manipulates and burns ISO 9660 filesystem images.
 
-   Copyright 2007-2010 Thomas Schmitt, <scdbackup@gmx.net>
+   Copyright 2007-2011 Thomas Schmitt, <scdbackup@gmx.net>
 
    Provided under GPL version 2 or later.
 
@@ -126,6 +126,11 @@ int Xorriso_graft_split(struct XorrisO *xorriso, IsoImage *volume,
  if(ret<0)
    return(ret);
  *node= (IsoNode *) new_dir;
+ if(xorriso->update_flags & 1) {
+   ret= Xorriso_mark_update_merge(xorriso, img_name, node, 1);
+   if(ret <= 0)
+     return(0);
+ }
  total_parts= size / xorriso->split_size;
  if(size % xorriso->split_size)
    total_parts++;
@@ -213,6 +218,11 @@ int Xorriso_tree_graft_node(struct XorrisO *xorriso, IsoImage *volume,
    if(ret <= 0)
      goto ex;
  }
+ if(xorriso->update_flags & 1) {
+   ret= Xorriso_mark_update_merge(xorriso, img_name, *node, 1);
+   if(ret <= 0)
+     goto ex;
+ }
 
 ex:;
  if(ret<0) {
@@ -611,20 +621,30 @@ int Xorriso_graft_in(struct XorrisO *xorriso, void *boss_iter,
  int target_is_split, hide_attrs;
  struct stat stbuf;
 
- ret= Xorriso_path_is_excluded(xorriso, disk_path, !(flag&4));
- if(ret<0)
-   return(ret);
- if(ret>0)
-   return(3*!!(flag&16));
-
- /* Check for mkisofs-style hidings */
  hide_attrs= (flag >> 8) & 3;
- if(hide_attrs != 3) {
-   ret= Xorriso_path_is_hidden(xorriso, disk_path, 0);
+ if (disk_path == NULL && !(flag & 1)) {
+   Xorriso_msgs_submit(xorriso, 0,
+         "Program error: Xorriso_graft_in(): disk_path == NULL && !(flag & 1)",
+         0, "ABORT", 0);
+    return(-1);
+ }
+ if (disk_path == NULL) {
+   disk_path= "";
+ } else {
+   ret= Xorriso_path_is_excluded(xorriso, disk_path, !(flag&4));
    if(ret<0)
      return(ret);
-   if(ret>=0)
-     hide_attrs|= ret;
+   if(ret>0)
+     return(3*!!(flag&16));
+
+   /* Check for mkisofs-style hidings */
+   if(hide_attrs != 3) {
+     ret= Xorriso_path_is_hidden(xorriso, disk_path, 0);
+     if(ret<0)
+       return(ret);
+     if(ret>=0)
+       hide_attrs|= ret;
+   }
  }
 
  for(cpt= img_path; 1; cpt++) {
@@ -638,10 +658,14 @@ int Xorriso_graft_in(struct XorrisO *xorriso, void *boss_iter,
  break;
  }
  if(cpt!=NULL) {
-   Xorriso_msgs_submit(xorriso, 0, disk_path, 0, "ERRFILE", 0);
+   if(disk_path[0])
+     Xorriso_msgs_submit(xorriso, 0, disk_path, 0, "ERRFILE", 0);
    sprintf(xorriso->info_text,
-         "Unsupported relative addressing in iso_rr_path %s (disk: %s)",
-         Text_shellsafe(img_path, sfe, 0), Text_shellsafe(disk_path, sfe2, 0));
+           "Unsupported relative addressing in iso_rr_path %s",
+           Text_shellsafe(img_path, sfe, 0));
+   if(disk_path[0])
+     sprintf(xorriso->info_text + strlen(xorriso->info_text),
+             " (disk: %s)", Text_shellsafe(disk_path, sfe, 0));
    Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, errno, "FAILURE", 0);
    return(0);
  }
@@ -740,7 +764,8 @@ int Xorriso_graft_in(struct XorrisO *xorriso, void *boss_iter,
            return(ret);
          if(ret==3) {
            sprintf(xorriso->info_text, "User revoked adding of: %s",
-                   Text_shellsafe(disk_path, sfe, 0));
+                   disk_path[0] ? Text_shellsafe(disk_path, sfe, 0)
+                                : Text_shellsafe(img_path, sfe, 0));
            Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "NOTE", 0);
            return(3*!!(flag&16));
          }
@@ -748,7 +773,8 @@ int Xorriso_graft_in(struct XorrisO *xorriso, void *boss_iter,
          goto handle_path_node;
        }
 
-       Xorriso_msgs_submit(xorriso, 0, disk_path, 0, "ERRFILE", 0);
+       if (disk_path[0])
+         Xorriso_msgs_submit(xorriso, 0, disk_path, 0, "ERRFILE", 0);
        sprintf(xorriso->info_text,
               "While grafting '%s' : '%s' exists and may not be overwritten",
               img_path, path);
@@ -763,7 +789,8 @@ handle_path_node:;
      ret= iso_tree_add_new_dir(dir, apt, &hdir);
      if(ret<0) {
        Xorriso_process_msg_queues(xorriso,0);
-       Xorriso_msgs_submit(xorriso, 0, disk_path, 0, "ERRFILE", 0);
+       if(disk_path[0])
+         Xorriso_msgs_submit(xorriso, 0, disk_path, 0, "ERRFILE", 0);
        Xorriso_report_iso_error(xorriso, img_path, ret,
                                 "Cannot create directory", 0, "FAILURE", 1);
        sprintf(xorriso->info_text,
@@ -771,13 +798,19 @@ handle_path_node:;
        Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "FAILURE", 0);
        return(0);
      }
+     if(xorriso->update_flags & 1) {
+       ret= Xorriso_mark_update_merge(xorriso, path, (IsoNode *) hdir, 1);
+       if(ret <= 0)
+         return(0);
+     }
+
      dir= hdir;
      Xorriso_set_change_pending(xorriso, 0);
      iso_node_set_ctime((IsoNode *) dir, time(NULL));
      iso_node_set_uid((IsoNode *) dir, geteuid());
      iso_node_set_gid((IsoNode *) dir, getegid());
 
-     if(disk_path!=NULL && !done)
+     if(disk_path[0] && !done)
        /* This not only copies disk directory properties
           but also sets eventual hide_attrs */
        Xorriso_copy_implicit_properties(xorriso, dir, img_path, path, disk_path,
@@ -899,6 +932,7 @@ unsupported_type:;
 
 
 /* @param flag bit0= do not produce info message on success
+               bit1= do not raise protest if directory already exists
    @return 1=success,
            0=was already directory, -1=was other type, -2=other error
 */
@@ -911,8 +945,10 @@ int Xorriso_mkdir(struct XorrisO *xorriso, char *path, int flag)
  if(ret<0)
    return(-2);
  if(ret>0) {
+   if(ret == 2 && (flag & 2))
+     return(0);
    sprintf(xorriso->info_text,"-mkdir: Address already existing %s",
-          Text_shellsafe(eff_path, sfe, 0));
+           Text_shellsafe(eff_path, sfe, 0));
    Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0,
                        (ret==2 ? "WARNING" : "FAILURE"), 0);
    return(-1+(ret==2));
@@ -1187,6 +1223,38 @@ ex:;
 } 
 
 
+int Xorriso_overwrite_dest(struct XorrisO *xorriso, void *boss_iter,
+                           char *eff_dest, int dest_ret, char *activity,
+                           int flag)
+{
+ int ret;
+ char sfe[5*SfileadrL];
+
+ if(dest_ret==2 && xorriso->do_overwrite!=1) {
+   sprintf(xorriso->info_text, "%s: May not overwrite directory: %s",
+           activity, Text_shellsafe(eff_dest, sfe, 0));
+   Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "FAILURE", 0);
+   return(0);
+ } else if (dest_ret==1 && !xorriso->do_overwrite) {
+   sprintf(xorriso->info_text, "%s: May not overwite: %s",
+           activity, Text_shellsafe(eff_dest, sfe, 0));
+   Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "FAILURE", 0);
+   return(0);
+ } else if(dest_ret>0) {
+   ret= Xorriso_rmi(xorriso, boss_iter, (off_t) 0, eff_dest, 1|8);
+   if(ret<=0)
+     return(0);
+   if(ret==3) {
+     sprintf(xorriso->info_text, "%s: User revoked removal of: %s",
+             activity, Text_shellsafe(eff_dest, sfe, 0));
+     Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "NOTE", 0);
+     return(0);
+   }
+ }
+ return(1);
+}
+
+
 /* @param boss_iter Opaque handle to be forwarded to actions in ISO image
                     Set to NULL if calling this function from outside ISO world
 */
@@ -1249,27 +1317,10 @@ int Xorriso_rename(struct XorrisO *xorriso, void *boss_iter,
  }
 
  /* Check whether destination exists and may be not overwriteable */
- if(dest_ret==2 && xorriso->do_overwrite!=1) {
-   sprintf(xorriso->info_text, "Renaming may not overwrite directory: %s",
-           Text_shellsafe(eff_dest, sfe, 0));
-   Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "FAILURE", 0);
-   return(0);
- } else if (dest_ret==1 && !xorriso->do_overwrite) {
-   sprintf(xorriso->info_text, "Renaming may not overwite: %s",
-           Text_shellsafe(eff_dest, sfe, 0));
-   Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "FAILURE", 0);
-   return(0);
- } else if(dest_ret>0) {
-   ret= Xorriso_rmi(xorriso, boss_iter, (off_t) 0, eff_dest, 1|8);
-   if(ret<=0)
-     return(0);
-   if(ret==3) {
-     sprintf(xorriso->info_text, "User revoked renaming of: %s",
-             Text_shellsafe(eff_origin, sfe, 0));
-     Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "NOTE", 0);
-     return(0);
-   }
- }
+ ret= Xorriso_overwrite_dest(xorriso, boss_iter,
+                             eff_dest, dest_ret, "Renaming", 0);
+ if(ret <= 0)
+   return(ret);
 
  /* Ensure existence of destination directory */
  strcpy(dir_adr, eff_dest);
@@ -1345,6 +1396,175 @@ int Xorriso_rename(struct XorrisO *xorriso, void *boss_iter,
 }
 
 
+int Xorriso_cannot_clone(struct XorrisO *xorriso, char *eff_origin,
+                         char *eff_dest, int iso_error, int flag)
+{
+ Xorriso_report_iso_error(xorriso, eff_dest, iso_error, "Cannot clone",
+                          0, "FAILURE", 1);
+ sprintf(xorriso->info_text, "Failed to clone ");
+ Text_shellsafe(eff_origin, xorriso->info_text, 1);
+ Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "FAILURE", 0);
+ return(0);
+}
+
+/* @param flag bit0= for iso_tree_clone() : merge directories
+               bit1= do not issue NOTE message
+*/
+int Xorriso_clone_tree(struct XorrisO *xorriso, void *boss_iter,
+                       char *origin, char *dest, int flag)
+{
+ int ret, dest_ret, l;
+ char eff_dest[SfileadrL], eff_origin[SfileadrL], dir_adr[SfileadrL];
+ char *leafname;
+ IsoImage *volume;
+ IsoDir *new_parent;
+ IsoNode *origin_node, *dir_node, *new_node;
+
+ ret= Xorriso_get_volume(xorriso, &volume, 0);
+ if(ret <= 0)
+   return(ret);
+
+ ret= Xorriso_normalize_img_path(xorriso, xorriso->wdi, origin, eff_origin, 0);
+ if(ret<=0)
+   return(ret);
+ ret= Xorriso_node_from_path(xorriso, volume, eff_origin, &origin_node, 0);
+ if(ret <= 0)
+   return(ret);
+
+ dest_ret= Xorriso_normalize_img_path(xorriso, xorriso->wdi, dest, eff_dest,1);
+ if(dest_ret<0)
+   return(dest_ret);
+ if(dest_ret > 0) {
+   if(eff_dest[0] == 0)
+     strcpy(eff_dest, "/");
+   sprintf(xorriso->info_text,
+           "Cloning: Copy address already exists: ");
+   Text_shellsafe(eff_dest, xorriso->info_text, 1);
+   Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "FAILURE", 0);
+   return(0);
+ } else {
+   /* obtain eff_dest address despite it does not exist */
+   ret= Xorriso_normalize_img_path(xorriso, xorriso->wdi, dest, eff_dest, 2);
+   if(ret<=0)
+     return(ret);
+ }
+
+ /* Obtain parent path and leaf name */
+ strcpy(dir_adr, eff_dest);
+ for(l= strlen(dir_adr); l > 0; ) {
+   if(dir_adr[l - 1] == '/')
+     dir_adr[--l]= 0;
+   else
+ break;
+ }
+ leafname= strrchr(dir_adr, '/');
+ if(leafname == NULL) {
+   leafname= dir_adr;
+   if (leafname[0] == 0) {
+     Xorriso_msgs_submit(xorriso, 0, "Empty file name as clone destination",
+                         0, "FAILURE", 0);
+     return(0);
+   }
+ } else {
+   *leafname= 0;
+   leafname++;
+   if(dir_adr[0] != 0) {
+     /* Ensure existence of destination directory */
+     ret= Xorriso_graft_in(xorriso, boss_iter, NULL, dir_adr,
+                           (off_t) 0, (off_t) 0, 1);
+     if(ret <= 0)
+       return(ret);
+   }
+ }
+
+ ret= Xorriso_node_from_path(xorriso, volume, dir_adr, &dir_node, 0);
+ if(ret <= 0)
+   return(ret);
+ new_parent= (IsoDir *) dir_node;
+
+ ret = iso_tree_clone(origin_node, new_parent, leafname, &new_node, flag & 1);
+ Xorriso_process_msg_queues(xorriso,0);
+ if(ret < 0) {
+   Xorriso_cannot_clone(xorriso, eff_origin, eff_dest, ret, 0);
+   return(0);
+ }
+ Xorriso_set_change_pending(xorriso, 0);
+ if(!(flag & 2)) {
+   strcpy(xorriso->info_text, "Cloned in ISO image: ");
+   Text_shellsafe(eff_origin, xorriso->info_text, 1);
+   strcat(xorriso->info_text, " to ");
+   Text_shellsafe(eff_dest, xorriso->info_text, 1);
+   strcat(xorriso->info_text, "\n");
+   Xorriso_info(xorriso, 0);
+ }
+ return(1);
+}
+
+
+int Xorriso_clone_under(struct XorrisO *xorriso, char *origin, char *dest,
+                        int flag)
+{
+ int ret, pass;
+ char eff_dest[SfileadrL], eff_origin[SfileadrL], *namept;
+ IsoDir *origin_dir, *dest_dir;
+ IsoDirIter *iter= NULL;
+ IsoNode *origin_node, *new_node;
+ IsoImage *volume;
+
+ ret= Xorriso_get_volume(xorriso, &volume, 0);
+ if(ret <= 0)
+   return(ret);
+ ret= Xorriso_dir_from_path(xorriso, "Copy source", origin, &origin_dir, 0);
+ if(ret <= 0)
+   return(ret);
+ ret= Xorriso_dir_from_path(xorriso, "Copy destination", dest, &dest_dir, 0);
+ if(ret <= 0)
+   return(ret);
+
+ for(pass= 0; pass < 2; pass++) {
+   ret= iso_dir_get_children(origin_dir, &iter);
+   if(ret < 0) {
+     Xorriso_cannot_create_iter(xorriso, ret, 0);
+     {ret= -1; goto ex;}
+   }
+   Xorriso_process_msg_queues(xorriso,0);
+
+   while(iso_dir_iter_next(iter, &origin_node) == 1) {
+     namept= (char *) iso_node_get_name(origin_node);
+     sprintf(eff_origin, "%s/%s", origin, namept);
+     sprintf(eff_dest, "%s/%s", dest, namept);
+     if(pass == 0) {
+       ret= Xorriso_node_from_path(xorriso, volume, eff_dest, &new_node, 1);
+       if(ret < 0)
+         goto ex;
+       if(ret > 0) {
+         sprintf(xorriso->info_text, "Cloning: Copy address already exists: ");
+         Text_shellsafe(eff_dest, xorriso->info_text, 1);
+         Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "FAILURE", 0);
+         ret= 0; goto ex;
+       }
+     } else {
+       ret = iso_tree_clone(origin_node, dest_dir, namept, &new_node, 1);
+       Xorriso_process_msg_queues(xorriso,0);
+       if(ret < 0) {
+         Xorriso_cannot_clone(xorriso, eff_origin, eff_dest, ret, 0);
+         ret= 0; goto ex;
+       }
+     }
+   }   
+   iso_dir_iter_free(iter);
+   iter= NULL;
+ }
+ Xorriso_set_change_pending(xorriso, 0);
+ ret= 1;
+ex:;
+ if(iter != NULL)
+   iso_dir_iter_free(iter);
+ Xorriso_process_msg_queues(xorriso,0);
+ return(ret);
+}
+
+
 int Xorriso_set_st_mode(struct XorrisO *xorriso, char *in_path,
                         mode_t mode_and, mode_t mode_or, int flag)
 {
@@ -1656,8 +1876,8 @@ int Xorriso_widen_hardlink(struct XorrisO *xorriso, void * boss_iter,
 
  /* >>> compare_result bit17 = is_split */;
 
- ret= Xorriso_update_interpreter(xorriso, boss_iter, compare_result, disk_path,
-                                 abs_path, 1);
+ ret= Xorriso_update_interpreter(xorriso, boss_iter, NULL,
+                                 compare_result, disk_path, abs_path, 1);
  if(ret <= 0)
    return(ret);
  return(ret);
@@ -1886,10 +2106,13 @@ int Xorriso_findi_action(struct XorrisO *xorriso, struct FindjoB *job,
    Findjob_set_start_path(subjob, abs_path, 0);
    ret= Xorriso_findi(xorriso, subjob, boss_iter, boss_mem, NULL,
                       abs_path, &dir_stbuf, depth, 1);
- } else if(action==14 || action==17) { /* compare , update */
+ } else if(action==14 || action==17 || action == 41) {
+                                          /* compare , update , update_merge */
    Findjob_get_start_path(job, &iso_prefix, 0);
-   ret= Xorriso_find_compare(xorriso, (void *) boss_iter, abs_path,
-                             iso_prefix, target, (action==17)|((flag&1)<<1));
+   ret= Xorriso_find_compare(xorriso, (void *) boss_iter, (void *) node,
+                         abs_path, iso_prefix, target,
+                         (action == 17 || action == 41)
+                         | ((flag&1)<<1) | ((action == 41) << 2));
    if(ret==2)
      deleted= 1;
    if(ret==3)
@@ -1970,6 +2193,17 @@ int Xorriso_findi_action(struct XorrisO *xorriso, struct FindjoB *job,
    if(ret > 0)
      ret= Xorriso_estimate_file_size(xorriso, job, basename, stbuf.st_mode,
                                      stbuf.st_size, 0);
+ } else if(action == 42) { /* rm_merge */
+   ret= Xorriso_mark_update_merge(xorriso, show_path, node, 2 | 4);
+   if(ret == 2) {
+     ret= Xorriso_rmi(xorriso, boss_iter, boss_mem, abs_path, 1|hflag);
+     sprintf(xorriso->info_text, "Deleted ");
+     Text_shellsafe(show_path, xorriso->info_text, 1);
+     Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "UPDATE", 0);
+     deleted= 1;
+   }
+ } else if(action == 43) { /* clear_merge */
+   ret= Xorriso_mark_update_merge(xorriso, show_path, node, 2 | 4);
  } else { /* includes : 15 in_iso */
    sprintf(xorriso->result_line, "%s\n", Text_shellsafe(show_path, sfe, 0));
    Xorriso_result(xorriso, 0);
@@ -2372,9 +2606,10 @@ int Xorriso_findi(struct XorrisO *xorriso, struct FindjoB *job,
  mem= boss_mem;
  hflag= 1;
  if(action==1 || action==2 || action==3 || action==17 || action == 28 ||
-    action == 32)
+    action == 32 || action == 41 || action == 42)
    hflag|= 2; /* need freedom to manipulate image */
- if(action==14 || action==17 || action == 28 || action == 35 || action == 36)
+ if(action==14 || action==17 || action == 28 || action == 35 || action == 36 ||
+    action == 41)
    hflag|= 4; /* need LBA sorted iteration for good data reading performance */
  ret= Xorriso_findi_iter(xorriso, dir_node, &mem,
                          &iter, &node_array, &node_count, &node_idx,
@@ -2786,3 +3021,92 @@ ex:;
  return(ret);
 }
 
+
+int Xorriso__mark_update_xinfo(void *data, int flag)
+{
+ /* data is an int disguised as pointer. It does not point to memory. */
+ return(1);
+}
+
+
+int Xorriso__mark_update_cloner(void *old_data, void **new_data, int flag)
+{
+ *new_data= NULL;
+ if(flag)
+   return(ISO_XINFO_NO_CLONE);
+ if(old_data == NULL)
+   return(0);
+ /* data is an int disguised as pointer. It does not point to memory. */
+ *new_data= old_data;
+ return(0);
+}
+
+
+/* @param flag bit0= found on disk
+               bit1= inquire visit-found status:
+                     1=not visited, 2=not found, 3=found
+               bit2= with bit1: delete xinfo before returning status
+*/
+int Xorriso_mark_update_merge(struct XorrisO *xorriso, char *path,
+                              void *in_node, int flag)
+{
+ int ret;
+ void *xipt= NULL;
+ IsoNode *node;
+
+ if(in_node == NULL) {
+   ret= Xorriso_node_from_path(xorriso, NULL, path, &node, 0);
+   if(ret <= 0)
+     return(ret);
+ } else
+   node= (IsoNode *) in_node;
+ ret= iso_node_get_xinfo(node, Xorriso__mark_update_xinfo, &xipt);
+ if(ret < 0) {
+   Xorriso_process_msg_queues(xorriso,0);
+   Xorriso_report_iso_error(xorriso, "", ret,
+                            "Error when looking for update_merge xinfo",
+                            0, "FAILURE", 1);
+   return(0);
+ }
+ if(flag & 2) { /* Inquire status and optionally delete xinfo */
+   if(ret == 0)
+     return(1);
+   if(flag & 4) {
+     ret= iso_node_remove_xinfo(node, Xorriso__mark_update_xinfo);
+     if(ret < 0) {
+       Xorriso_process_msg_queues(xorriso,0);
+       Xorriso_report_iso_error(xorriso, "", ret,
+                                "Error when removing update_merge xinfo",
+                                0, "FAILURE", 1);
+       return(0);
+     }
+   }
+   if(((char *) &xipt)[0])
+     return(3);
+   return(2);
+ }
+ /* xipt is a byte value disguised as void pointer */
+ if(ret == 1) {
+   if(((char *) &xipt)[0])
+     return(1);
+   if(!(flag & 1))
+     return(1);
+ } else
+   ((char *) &xipt)[0]= 0;
+ if(flag & 1)
+   ((char *) &xipt)[0]= 1;
+ ret= iso_node_remove_xinfo(node, Xorriso__mark_update_xinfo);
+ if(ret < 0)
+   goto set_error;
+ ret= iso_node_add_xinfo(node, Xorriso__mark_update_xinfo, xipt);
+ if(ret <= 0) {
+set_error:;
+   Xorriso_process_msg_queues(xorriso,0);
+   Xorriso_report_iso_error(xorriso, "", ret,
+                            "Error when trying to set update_merge xinfo",
+                            0, "FAILURE", 1);
+   return(0);
+ }
+ return(1);
+}
+
