@@ -1,7 +1,7 @@
 
 /* xorriso - creates, loads, manipulates and burns ISO 9660 filesystem images.
 
-   Copyright 2007-2010 Thomas Schmitt, <scdbackup@gmx.net>
+   Copyright 2007-2011 Thomas Schmitt, <scdbackup@gmx.net>
 
    Provided under GPL version 2 or later.
 
@@ -23,7 +23,14 @@
 #include <sys/time.h>
 #include <time.h>
 #include <errno.h>
+
+#ifdef HAVE_STDINT_H
 #include <stdint.h>
+#else
+#ifdef HAVE_INTTYPES_H
+#include <inttypes.h>
+#endif
+#endif
 
 /* for -charset */
 #include <iconv.h>
@@ -48,6 +55,7 @@
 #include "xorrisoburn.h"
 
 #include "lib_mgt.h"
+#include "iso_manip.h"
 
 
 int Xorriso_abort(struct XorrisO *xorriso, int flag)
@@ -196,6 +204,9 @@ LIBISOBURN_MISCONFIGURATION_ = 0;
        zisofs_ctrl.compression_level;
  }
 
+ iso_node_xinfo_make_clonable(Xorriso__mark_update_xinfo,
+                              Xorriso__mark_update_cloner, 0);
+
  /* Second initialization. This time with libs. */
  Xorriso_preparer_string(xorriso, xorriso->preparer_id, 0);
 
