@@ -451,7 +451,7 @@ int Xorriso_option_map(struct XorrisO *xorriso, char *disk_path,
 int Xorriso_option_map_l(struct XorrisO *xorriso, int argc, char **argv,
                          int *idx, int flag)
 {
- int ret, end_idx, optc= 0, was_failure= 1, i, fret, mode;
+ int ret, end_idx, optc= 0, was_failure= 1, i, fret, mode, problem_count;
  int ns_flag= 2|4, nt_flag= 2, opt_args_flag= 2, made_di_array= 0;
  char source_prefix[SfileadrL], target_prefix[SfileadrL], *cmd, **optv= NULL;
  char eff_source[SfileadrL], eff_target[SfileadrL], *source_pt, *s_wd, *t_wd;
@@ -576,7 +576,10 @@ int Xorriso_option_map_l(struct XorrisO *xorriso, int argc, char **argv,
                                   source_prefix, strlen(source_prefix) + 1, 0);
    if(ret <= 0)
      goto ex;
-   ret= Xorriso_restore_sorted(xorriso, optc, eff_src_array, eff_tgt_array, 0);
+   ret= Xorriso_restore_sorted(xorriso, optc, eff_src_array, eff_tgt_array,
+                               &problem_count, 0);
+   if(ret <= 0 || problem_count > 0)
+     was_failure= 1;
  }
  if(mode==0)
    Xorriso_pacifier_callback(xorriso, "files added", xorriso->pacifier_count,
