@@ -181,9 +181,9 @@ no_memory:;
 
  ret= 1;
 ex:;
+ if(patterns!=NULL)
+   free((char *) patterns);
  if(ret<=0) {
-   if(patterns!=NULL)
-     free((char *) patterns);
    Sfile_destroy_argv(&filec, &filev, 0);
  } else {
    *optc= filec;
@@ -314,8 +314,9 @@ int Xorriso_cpmv_args(struct XorrisO *xorriso, char *cmd,
  if(ret==2 || ((flag&1) && *optc > 1 && ret==0)) {
    is_dir= 1;
  } else if(*optc > 1) {
-   for(i= 0; i<*optc; i++)
-     Xorriso_msgs_submit(xorriso, 0, (*optv)[i], 0, "ERRFILE", 0);
+   if(flag & 2)
+     for(i= 0; i<*optc; i++)
+       Xorriso_msgs_submit(xorriso, 0, (*optv)[i], 0, "ERRFILE", 0);
    sprintf(xorriso->info_text,
         "%s: more than one origin given, destination is a non-directory: %s",
         cmd, Text_shellsafe(destv[0], sfe, 0));
@@ -477,7 +478,7 @@ int Xorriso_count_args(struct XorrisO *xorriso, int argc, char **argv,
     ""
  };
  static char arg2_commands[][40]= {
-    "assert_volid","boot_image","compare","compare_r","drive_class",
+    "assert_volid","boot_image","clone","compare","compare_r","drive_class",
     "errfile_log","error_behavior","extract","extract_single",
     "jigdo","load","logfile",
     "map","map_single","page","return_with",
@@ -497,7 +498,7 @@ int Xorriso_count_args(struct XorrisO *xorriso, int argc, char **argv,
     "check_md5","check_md5_r","check_media","check_media_defaults",
     "chgrp","chgrpi","chgrp_r","chgrp_ri","chmod","chmodi",
     "chmod_r","chmod_ri","chown","chowni","chown_r","chown_ri",
-    "compare_l","cpr","cpri","cp_rax","cp_rx","cpax","cpx",
+    "compare_l","cp_clone","cp_rax","cp_rx","cpr","cpri", "cpax","cpx",
     "du","dui","dus","dusi","dux","dusx","external_filter","extract_l",
     "file_size_limit","find","findi","finds","findx",
     "getfacl","getfacli","getfacl_r","getfacl_ri",
@@ -748,6 +749,10 @@ next_command:;
    (*idx)+= 1;
    ret= Xorriso_option_chowni(xorriso, arg1, argc, argv, idx, 0);   
 
+ } else if(strcmp(cmd,"clone")==0) {
+   (*idx)+= 2;
+   ret= Xorriso_option_clone(xorriso, arg1, arg2, 1);
+
  } else if(strcmp(cmd,"close")==0) {
    (*idx)++;
    ret= Xorriso_option_close(xorriso, arg1, 0);
@@ -781,13 +786,16 @@ next_command:;
    (*idx)++;
    Xorriso_option_copyright_file(xorriso, arg1, 0);
 
- } else if(strcmp(cmd,"cpr")==0 || strcmp(cmd,"cpri")==0) {
-   ret= Xorriso_option_cpri(xorriso, argc, argv, idx, 0);
+ } else if(strcmp(cmd,"cp_clone") == 0) {
+   ret= Xorriso_option_cp_clone(xorriso, argc, argv, idx, 0);
 
  } else if(strcmp(cmd,"cp_rx")==0 || strcmp(cmd,"cp_rax")==0) {
    ret= Xorriso_option_cpx(xorriso, argc, argv, idx,
                            1|((strcmp(cmd,"cp_rax")==0)<<1));
 
+ } else if(strcmp(cmd,"cpr")==0 || strcmp(cmd,"cpri")==0) {
+   ret= Xorriso_option_cpri(xorriso, argc, argv, idx, 0);
+
  } else if(strcmp(cmd,"cpx")==0 || strcmp(cmd,"cpax")==0) {
    ret= Xorriso_option_cpx(xorriso, argc, argv, idx,
                            (strcmp(cmd,"cpax")==0)<<1);
@@ -1312,7 +1320,6 @@ next_command:;
    ret= Xorriso_option_temp_mem_limit(xorriso, arg1, 0);
 
  } else if(strcmp(cmd,"test")==0) { /* This option does not exist. */
-
    /* install temporary test code here */;
 
  } else if(strcmp(cmd,"toc")==0) {
