@@ -2,7 +2,7 @@
 
 /* xorriso - creates, loads, manipulates and burns ISO 9660 filesystem images.
 
-   Copyright 2007-2010 Thomas Schmitt, <scdbackup@gmx.net>
+   Copyright 2007-2011 Thomas Schmitt, <scdbackup@gmx.net>
 
    Provided under GPL version 2 or later.
 
@@ -25,7 +25,14 @@
 #include <errno.h>
 
 #include <fcntl.h>
+
+#ifdef HAVE_STDINT_H
 #include <stdint.h>
+#else
+#ifdef HAVE_INTTYPES_H
+#include <inttypes.h>
+#endif
+#endif
 
 #ifdef Xorriso_standalonE
 
@@ -741,6 +748,7 @@ int Xorriso_write_session(struct XorrisO *xorriso, int flag)
       ((!!(xorriso->do_md5 & 2)) * isoburn_igopt_session_md5) |
       ((!!(xorriso->do_md5 & 4)) * isoburn_igopt_file_md5) |
       ((!!(xorriso->do_md5 & 8)) * isoburn_igopt_file_stability) |
+      ((!!xorriso->do_old_empty) * isoburn_igopt_old_empty) |
       ((flag & 1) * isoburn_igopt_will_cancel);
  if(xorriso->no_emul_toc & 1)
    ext|= isoburn_igopt_no_emul_toc;
@@ -1207,6 +1215,7 @@ int Xorriso_pacifier_loop(struct XorrisO *xorriso, struct burn_drive *drive,
    break;
    }
  }
+ iso_image_unref(image);
  return(1);
 }
 
@@ -1998,6 +2007,12 @@ int Xorriso_relax_compliance(struct XorrisO *xorriso, char *mode,
      xorriso->relax_compliance= was;
      return(0);
 
+   } else if((l ==  9 && strncmp(cpt, "old_empty", l) == 0) ||
+             (l == 12 && strncmp(cpt, "old_empty_on", l) == 0)) {
+     xorriso->do_old_empty= 1;
+   } else if(l == 13 && strncmp(cpt, "old_empty_off", l) == 0) {
+     xorriso->do_old_empty= 0;
+
    } else {
      if(l<SfileadrL)
        sprintf(xorriso->info_text, "-compliance: unknown rule '%s'",
@@ -2070,6 +2085,8 @@ int Xorriso_get_relax_text(struct XorrisO *xorriso, char mode[1024],
            xorriso->untranslated_name_len);
  if(xorriso->do_iso1999)
    sprintf(mode + strlen(mode), ":iso_9660_1999");
+ if(xorriso->do_old_empty)
+   sprintf(mode + strlen(mode), ":old_empty");
  return(1 +
        (r == Xorriso_relax_compliance_defaulT && !(xorriso->no_emul_toc & 1)
         && xorriso->untranslated_name_len == 0 && !xorriso->do_iso1999 &&
