@@ -61,7 +61,7 @@ struct XorrisO;
 */
 #define Xorriso_header_version_majoR  1
 #define Xorriso_header_version_minoR  0
-#define Xorriso_header_version_micrO  0
+#define Xorriso_header_version_micrO  2
 
 
 /** Eventually something like ".pl01" to indicate a bug fix. Normally empty.
@@ -703,6 +703,10 @@ int Xorriso_option_chmodi(struct XorrisO *xorriso, char *mode,
 int Xorriso_option_chowni(struct XorrisO *xorriso, char *uid,
                           int argc, char **argv, int *idx, int flag);
 
+/* Option -clone */
+int Xorriso_option_clone(struct XorrisO *xorriso, char *origin, char *dest,
+                         int flag);
+
 /* Option -close "on"|"off" */
 int Xorriso_option_close(struct XorrisO *xorriso, char *mode, int flag);
 
@@ -738,6 +742,10 @@ int Xorriso_option_compliance(struct XorrisO *xorriso, char *mode, int flag);
 int Xorriso_option_copyright_file(struct XorrisO *xorriso, char *name,
                                   int flag);
 
+/* Option -cp_clone */
+int Xorriso_option_cp_clone(struct XorrisO *xorriso, int argc, char **argv,
+                            int *idx, int flag);
+
 /* Option -cpr alias -cpri */
 int Xorriso_option_cpri( struct XorrisO *xorriso, int argc, char **argv,
                          int *idx, int flag);
