@@ -20,7 +20,13 @@
 
 
 /* for uint32_t */
+#ifdef HAVE_STDINT_H
 #include <stdint.h>
+#else
+#ifdef HAVE_INTTYPES_H
+#include <inttypes.h>
+#endif
+#endif
 
 
 /** The source code release timestamp */
@@ -150,6 +156,11 @@ struct XorrisO { /* the global context of xorriso */
                            write first session to LBA 0 rather than 32.
                   */
 
+ int do_old_empty; /* See -compliance old_empty
+                      own data content: range [0,31]. The new way is to have
+                      a dedicated block to which all such files will point.
+                    */
+
  char scdbackup_tag_name[81];
  char scdbackup_tag_time[19];
  char scdbackup_tag_written[512];
@@ -571,6 +582,10 @@ struct XorrisO { /* the global context of xorriso */
 
  struct PermiteM *perm_stack; /* Temporarily altered dir access permissions */
 
+ /* bit0= update_merge active: mark all newly added nodes as visited+found
+ */
+ int update_flags;
+
  /* result (stdout, R: ) */
  char result_line[10*SfileadrL];
  int result_line_counter;
