@@ -19,7 +19,7 @@
 */
 #define xorriso_libisoburn_req_major  1
 #define xorriso_libisoburn_req_minor  0
-#define xorriso_libisoburn_req_micro  0
+#define xorriso_libisoburn_req_micro  2
 
 
 struct SpotlisT;          /* List of intervals with different read qualities */
@@ -112,6 +112,9 @@ int Xorriso_blank_as_needed(struct XorrisO *xorriso, int flag);
                bit1= remove empty directory: rmdir
                bit2= recursion: do not reassure in mode 2 "tree"
                bit3= this is for overwriting and not for plain removal
+               bit4= count deleted files in xorriso->pacifier_count
+               bit5= with bit0 only remove directory content, not the directory
+               bit6= do not delete eventually existing node from di_array
    @return   <=0 = error
                1 = removed simple node
                2 = removed directory or tree
@@ -120,6 +123,7 @@ int Xorriso_blank_as_needed(struct XorrisO *xorriso, int flag);
 int Xorriso_rmi(struct XorrisO *xorriso, void *boss_iter, off_t boss_mem,
                 char *path, int flag);
 
+
 /* @param flag bit0= long format
                bit1= do not print count of nodes
                bit2= du format
@@ -556,5 +560,15 @@ int Xorriso_jigdo_interpreter(struct XorrisO *xorriso, char *aspect, char *arg,
 int Xorriso_estimate_file_size(struct XorrisO *xorriso, struct FindjoB *job,
                       char *basename, mode_t st_mode, off_t st_size, int flag);
 
+
+int Xorriso_clone_tree(struct XorrisO *xorriso, void *boss_iter,
+                       char *origin, char *dest, int flag);
+
+int Xorriso_clone_under(struct XorrisO *xorriso, char *origin, char *dest,
+                        int flag);
+
+int Xorriso_mark_update_merge(struct XorrisO *xorriso, char *path,
+                              void *node, int flag);
+
 #endif /* Xorrisoburn_includeD */
 
